# frozen_string_literal: true

module Bulkrax
  class CrcFolderComplexSubjectEntry < CrcFolderEntry
    include ComplexHelper
    def build
      begin
        raise Crc1280ExperimentCreatedError unless crc1280_experiment_created?

        complex_subject = ComplexSubject.new(self.raw_metadata.slice('subject_species', 'subject_type', 'subject_sex', 'subject_age'))
        complex_subject.subject_title = validate_and_fix_title
        complex_subject.source_identifier = identifier
        complex_subject.parent_source_identifier = self.raw_metadata['parents']
        complex_subject.is_imported = true
        complex_subject.save!
      rescue RSolr::Error::Http, Crc1280ExperimentCreatedError => e
        raise e
      rescue StandardError => e
        status_info(e)
      else
        status_info
      ensure
        self.save!
      end
    end

    def factory_class
      ComplexSubject
    end

    def crc1280_experiment_created?
      ::Crc1280Experiment.find_by_property(source: [self.raw_metadata['parents']]).present?
    end

    def validate_and_fix_title
      subject_title = sanitize_title(self.raw_metadata['title'])
      subject_title = subject_title.rjust(3, '0')  if subject_title.length < 3
      subject_title
    end
  end
end
